#!/usr/bin/perl
#=======================================================================
# FileName : cmd_InterProcessCom.pl
# Summary  : プロセス間通信ベースコマンド
# ----------------------------------------------------------------------
# DATE       NAME               REASON
# ---------- ------------------ ----------------------------------------
# 2015/03/06 Kiyoshi Oishi      Initial Release
#=======================================================================
use strict;
use JSON;
use Encode;
use Data::Dumper;
use IO::Socket::INET;
use Socket;

use lib '../com-lib';
use ComConst;
use ComFunc;
use CmdComFunc;
use ComLog;

our %messages = ();
my $ret = ComConst::STS_OK;

$ret = &main(@ARGV);

exit $ret;

#=======================================================================
# FuncName : main
# Summary  : プロセス間通信メイン処理
# Argument : (IN)  $reqFile  : リクエストファイル名(scalar)
#          : (IN)  $resFile  : レスポンスファイル名(scalar)
# Return   : 正常:0, 異常:-1
# ----------------------------------------------------------------------
# DATE       NAME               REASON
# ---------- ------------------ ----------------------------------------
# 2015/03/06 Kiyoshi Oishi      Initial Release
#=======================================================================
sub main {
    my $rc = ComConst::STS_OK;
    my $reqFile   = $_[0];
    my $resFile   = $_[1];
    my $requestData = "";
    my $json = "";
    my %responseData = ();
    my %conf      = ();
    my $logLevel = "";
    my $logmsg = "";
    my %procResponse = (
        status  => ComConst::STS_OK, 
        data    => (),
        message => ""
    );
    $responseData{ComConst::JSON_KEY_STS} = ComConst::STS_OK;
    $responseData{ComConst::JSON_KEY_MSG} = "";
    $responseData{ComConst::JSON_KEY_DAT} = ();
    my $key = "";

    # 初期化
    if($rc == ComConst::STS_OK){
        $rc = CmdComFunc::initializeBaseCmd($0, \%conf, \%messages);
    }

    # 開始ログ出力
    ComLog::log_output(ComLog::LOG_DEBUG, $messages{CM000D001});

    # リクエスト読み込み
    if($rc == ComConst::STS_OK){
        $requestData = CmdComFunc::readRequestFile($reqFile);
    }

    # 入力チェック
    if($rc == ComConst::STS_OK){
    }

    # AileunのTcpソケットにメッセージを送り
    # Aileun本体と通信を行う
    if($rc == ComConst::STS_OK){
        aileunProcCom($requestData->[0], \%conf, \%procResponse);
        $responseData{ComConst::JSON_KEY_STS} = $procResponse{status};
        $responseData{ComConst::JSON_KEY_DAT} = $procResponse{data};
        $responseData{ComConst::JSON_KEY_MSG} = $procResponse{message};
    }

    # レスポンスファイル出力
    $rc = CmdComFunc::writeResponseFile($resFile, \%responseData);

    # 終了ログ出力
    ComLog::log_output(ComLog::LOG_DEBUG, $messages{CM000D002});

    return $rc;
}

#=======================================================================
# FuncName : aileunProcCom
# Summary  : Aileunとの通信処理
# Argument : (IN)  $id                : リクエストID(scalar)
#          : (IN)  $ref_conf          : 設定情報(ref_associate)
#          : (OUT) $ref_proc_Response : 通信結果(ref_associate)
# Return   : Nothing
# ----------------------------------------------------------------------
# DATE       NAME               REASON
# ---------- ------------------ ----------------------------------------
# 2015/03/06 Kiyoshi Oishi      Initial Release
#=======================================================================
sub aileunProcCom {
    my $data         = $_[0];
    my $ref_conf     = $_[1];
    my $ref_procResponse = $_[2];
    my $sendMessage = "";

    # 送信メッセージ作成
    $sendMessage = sprintf(ComConst::AILEUN_SEND_MSG, $data->{btnId}, $data->{reqType}, $data->{targetView}, $data->{reqData}, $data->{auth});

    # Aileunとプロセス間通信
    tcpSocketComMain($sendMessage, $ref_conf->{TCP_PORT}, $ref_conf->{TIME_OUT}, $ref_procResponse);
}

#=======================================================================
# FuncName : tcpSocketComMain
# Summary  : TCPソケット通信処理
# Argument : (IN)  $sendMessage      : 送信メッセージ(scalar)
#          : (IN)  $tcpPort          : Tcpポート番号(scalar)
#          : (IN)  $timeOut          : タイムアウト時間(scalar)
#          : (OUT) $ref_procResponse : 通信結果(ref_associate)
# Return   : Nothing
# ----------------------------------------------------------------------
# DATE       NAME               REASON
# ---------- ------------------ ----------------------------------------
# 2015/03/06 Kiyoshi Oishi      Initial Release
#=======================================================================
sub tcpSocketComMain {
    my $sendMessage      = $_[0];
    my $tcpPort          = $_[1];
    my $timeOut          = $_[2];
    my $ref_procResponse = $_[3];
    my $server;
    my $recvMessage = "";
    my $rv;

    # ソケットの作成 & 接続
    $server = IO::Socket::INET->new(
        PeerAddr => "localhost",
        PeerPort => $tcpPort,
        Prot => "tcp",
        Timeout => $timeOut,
    );

    if(!$server){
        $ref_procResponse->{status} = ComConst::STS_NG;
    }

    # Aileunにメッセージ送信
    if($server){
        if($sendMessage eq ""){
            # 空文字だとサーバからの応答が返ってこないため
            # メッセージがからの場合は半角スペースを送信
            $sendMessage = " ";
        }
        $server->send($sendMessage);
    }

    # Aileunからの応答受信
    if($server){
        $server->recv($recvMessage, 4096);
        parseProcResponse($recvMessage, $ref_procResponse);
    }

    # 切断
    if($server){
        $server->close();
    }
}

#=======================================================================
# FuncName : parseProcResponse
# Summary  : Aileunから受信したメッセージを解析
# Argument : (IN)  $recvMessage      : 受信メッセージ(scalar)
#          : (OUT) $ref_procResponse : 通信結果(ref_associate)
# Return   : 変換後のデータ
# ----------------------------------------------------------------------
# DATE       NAME               REASON
# ---------- ------------------ ----------------------------------------
# 2015/03/12 Kiyoshi Oishi      Initial Release
#=======================================================================
sub parseProcResponse {
    my $recvMessage      = $_[0];
    my $ref_procResponse = $_[1];

    my $parseRes = decode_json($recvMessage);

    $ref_procResponse->{status}  = $parseRes->{status};
    $ref_procResponse->{data}    = $parseRes->{data};
    $ref_procResponse->{message} = $parseRes->{message};
}
